#version 430

layout(binding = 0, std140) uniform UBO
{
    mat4 MVP;
    vec4 SourceSize;
    vec4 OriginalSize;
    vec4 OutputSize;
    uint FrameCount;
    float bloom;
    float halation;
    float scans;
    float gamma_c;
    float smart_ei;
    float ei_limit;
    float sth;
    float overscanX;
    float overscanY;
    float c_shape;
    float intres;
    float prescalex;
    float scan_falloff;
    float rolling_scan;
    float bloom_dist;
} global;

struct Push
{
    float TATE;
    float IOS;
    float OS;
    float BLOOM;
    float brightboost;
    float brightboost1;
    float gsl;
    float scanline1;
    float scanline2;
    float beam_min;
    float beam_max;
    float beam_size;
    float h_sharp;
    float s_sharp;
    float csize;
    float bsize1;
    float warpX;
    float warpY;
    float glow;
    float vertmask;
    float spike;
};

uniform Push params;

layout(binding = 3) uniform sampler2D AvgLumPass;
layout(binding = 2) uniform sampler2D LinearizePass;
layout(binding = 4) uniform sampler2D PrePass;

layout(location = 0) in vec2 vTexCoord;
layout(location = 0) out vec4 FragColor;

vec2 Overscan(inout vec2 pos, float dx, float dy)
{
    pos = (pos * 2.0) - vec2(1.0);
    pos *= vec2(dx, dy);
    return (pos * 0.5) + vec2(0.5);
}

vec2 Warp(inout vec2 pos)
{
    pos = (pos * 2.0) - vec2(1.0);
    pos = mix(pos, vec2(pos.x * inversesqrt(1.0 - ((global.c_shape * pos.y) * pos.y)), pos.y * inversesqrt(1.0 - ((global.c_shape * pos.x) * pos.x))), vec2(params.warpX, params.warpY) / vec2(global.c_shape));
    return (pos * 0.5) + vec2(0.5);
}

float st(float x)
{
    return exp2(((-10.0) * x) * x);
}

float sw0(float x, float color, float scanline)
{
    float tmp = mix(params.beam_min, params.beam_max, color);
    float ex = x * tmp;
    float _89;
    if (params.gsl > (-0.5))
    {
        _89 = ex * ex;
    }
    else
    {
        _89 = mix(ex * ex, (ex * ex) * ex, 0.4000000059604644775390625);
    }
    ex = _89;
    return exp2((-scanline) * ex);
}

float sw1(inout float x, float color, float scanline)
{
    x = mix(x, params.beam_min * x, max(x - (0.4000000059604644775390625 * color), 0.0));
    float tmp = mix(1.2000000476837158203125 * params.beam_min, params.beam_max, color);
    float ex = x * tmp;
    return exp2(((-scanline) * ex) * ex);
}

float sw2(float x, float color, float scanline)
{
    float tmp = mix((2.5 - (0.5 * color)) * params.beam_min, params.beam_max, color);
    tmp = mix(params.beam_max, tmp, pow(x, color + 0.300000011920928955078125));
    float ex = x * tmp;
    return exp2(((-scanline) * ex) * ex);
}

vec3 gc(vec3 c)
{
    float mc = max(max(c.x, c.y), c.z);
    float mg = pow(mc, 1.0 / global.gamma_c);
    return (c * mg) / vec3(mc + 1.0000000133514319600180897396058e-10);
}

float st1(float x)
{
    return exp2(((-7.0) * x) * x);
}

void main()
{
    vec4 SourceSize = global.OriginalSize * mix(vec4(global.prescalex, 1.0, 1.0 / global.prescalex, 1.0), vec4(1.0, global.prescalex, 1.0, 1.0 / global.prescalex), vec4(params.TATE));
    float lum = texture(AvgLumPass, vec2(0.5)).w;
    float gamma_in = 1.0 / texture(LinearizePass, vec2(0.25)).w;
    float intera = texture(LinearizePass, vec2(0.75, 0.25)).w;
    bool interb = intera < 0.5;
    bool notate = params.TATE < 0.5;
    float SourceY = mix(SourceSize.y, SourceSize.x, params.TATE);
    float sy = 1.0;
    if (global.intres == 0.5)
    {
        sy = SourceY / 224.0;
    }
    else
    {
        if (global.intres == 1.0)
        {
            sy = SourceY / 240.0;
        }
        else
        {
            if (global.intres > 1.25)
            {
                sy = global.intres;
            }
        }
    }
    if (notate)
    {
        SourceSize *= vec4(1.0, 1.0 / sy, 1.0, sy);
    }
    else
    {
        SourceSize *= vec4(1.0 / sy, 1.0, sy, 1.0);
    }
    vec2 texcoord = vTexCoord;
    if ((params.IOS > 0.0) && (!interb))
    {
        vec2 ofactor = global.OutputSize.xy / SourceSize.xy;
        vec2 _420;
        if (params.IOS < 2.5)
        {
            _420 = floor(ofactor);
        }
        else
        {
            _420 = ceil(ofactor);
        }
        vec2 intfactor = _420;
        vec2 diff = ofactor / intfactor;
        float scan = mix(diff.y, diff.x, params.TATE);
        vec2 param = texcoord;
        float param_1 = scan;
        float param_2 = scan;
        vec2 _447 = Overscan(param, param_1, param_2);
        texcoord = _447;
        bool _450 = params.IOS == 1.0;
        bool _458;
        if (!_450)
        {
            _458 = params.IOS == 3.0;
        }
        else
        {
            _458 = _450;
        }
        if (_458)
        {
            texcoord = mix(vec2(vTexCoord.x, texcoord.y), vec2(texcoord.x, vTexCoord.y), vec2(params.TATE));
        }
    }
    float factor = (1.0 + (((1.0 - (0.5 * params.OS)) * params.BLOOM) / 100.0)) - ((lum * params.BLOOM) / 100.0);
    vec2 param_3 = texcoord;
    float param_4 = factor;
    float param_5 = factor;
    vec2 _499 = Overscan(param_3, param_4, param_5);
    texcoord = _499;
    vec2 param_6 = texcoord;
    float param_7 = (SourceSize.x - global.overscanX) / SourceSize.x;
    float param_8 = (SourceSize.y - global.overscanY) / SourceSize.y;
    vec2 _522 = Overscan(param_6, param_7, param_8);
    texcoord = _522;
    vec2 param_9 = texcoord;
    vec2 _526 = Warp(param_9);
    vec2 pos = _526;
    bool smarte = (global.smart_ei > 0.00999999977648258209228515625) && notate;
    vec2 coffset = vec2(0.5);
    vec2 ps = SourceSize.zw;
    vec2 OGL2Pos = (pos * SourceSize.xy) - coffset;
    vec2 fp = fract(OGL2Pos);
    vec2 dx = vec2(ps.x, 0.0);
    vec2 dy = vec2(0.0, ps.y);
    vec2 x2 = dx * 2.0;
    vec2 y2 = dy * 2.0;
    vec2 offx = dx;
    vec2 off2 = x2;
    vec2 offy = dy;
    float fpx = fp.x;
    if (!notate)
    {
        offx = dy;
        off2 = y2;
        offy = dx;
        fpx = fp.y;
    }
    float _582;
    if (notate)
    {
        _582 = fp.y;
    }
    else
    {
        _582 = fp.x;
    }
    float f = _582;
    vec2 pC4 = (floor(OGL2Pos) * ps) + (ps * 0.5);
    if (interb)
    {
        pC4.y = pos.y;
    }
    float zero = exp2(-params.h_sharp);
    float sharp1 = params.s_sharp * zero;
    float fdivider = min(global.prescalex, 2.0);
    float wl3 = (2.0 + fpx) / fdivider;
    float wl2 = (1.0 + fpx) / fdivider;
    float wl1 = fpx / fdivider;
    float wr1 = (1.0 - fpx) / fdivider;
    float wr2 = (2.0 - fpx) / fdivider;
    float wr3 = (3.0 - fpx) / fdivider;
    wl3 *= wl3;
    wl3 = exp2((-params.h_sharp) * wl3);
    wl2 *= wl2;
    wl2 = exp2((-params.h_sharp) * wl2);
    wl1 *= wl1;
    wl1 = exp2((-params.h_sharp) * wl1);
    wr1 *= wr1;
    wr1 = exp2((-params.h_sharp) * wr1);
    wr2 *= wr2;
    wr2 = exp2((-params.h_sharp) * wr2);
    wr3 *= wr3;
    wr3 = exp2((-params.h_sharp) * wr3);
    float fp1 = 1.0 - fpx;
    float twl3 = max(wl3 - sharp1, 0.0);
    float twl2 = max(wl2 - sharp1, mix(-0.119999997317790985107421875, 0.0, 1.0 - (fp1 * fp1)));
    float twl1 = max(wl1 - sharp1, 0.0);
    float twr1 = max(wr1 - sharp1, 0.0);
    float twr2 = max(wr2 - sharp1, mix(-0.119999997317790985107421875, 0.0, 1.0 - (fpx * fpx)));
    float twr3 = max(wr3 - sharp1, 0.0);
    bool sharp = sharp1 > 0.0;
    vec3 c1;
    vec3 c2;
    if (smarte)
    {
        twl3 = 0.0;
        twr3 = 0.0;
        c1 = texture(AvgLumPass, pC4).xyz;
        c2 = texture(AvgLumPass, pC4 + offy).xyz;
        c1 = max(c1 - vec3(global.sth), vec3(0.0));
        c2 = max(c2 - vec3(global.sth), vec3(0.0));
    }
    vec3 l3 = texture(LinearizePass, pC4 - off2).xyz;
    vec3 l2 = texture(LinearizePass, pC4 - offx).xyz;
    vec3 l1 = texture(LinearizePass, pC4).xyz;
    vec3 r1 = texture(LinearizePass, pC4 + offx).xyz;
    vec3 r2 = texture(LinearizePass, pC4 + off2).xyz;
    vec3 r3 = texture(LinearizePass, (pC4 + offx) + off2).xyz;
    vec3 colmin = min(min(l1, r1), min(l2, r2));
    vec3 colmax = max(max(l1, r1), max(l2, r2));
    if (smarte)
    {
        float pc = min(global.smart_ei * c1.y, global.ei_limit);
        float pl = min(global.smart_ei * max(c1.y, c1.x), global.ei_limit);
        float pr = min(global.smart_ei * max(c1.y, c1.z), global.ei_limit);
        twl1 = max(wl1 - pc, 0.00999999977648258209228515625 * wl1);
        twr1 = max(wr1 - pc, 0.00999999977648258209228515625 * wr1);
        twl2 = max(wl2 - pl, 0.00999999977648258209228515625 * wl2);
        twr2 = max(wr2 - pr, 0.00999999977648258209228515625 * wr2);
    }
    vec3 color1 = ((((((l3 * twl3) + (l2 * twl2)) + (l1 * twl1)) + (r1 * twr1)) + (r2 * twr2)) + (r3 * twr3)) / vec3(((((twl3 + twl2) + twl1) + twr1) + twr2) + twr3);
    if (sharp)
    {
        color1 = clamp(color1, colmin, colmax);
    }
    float ts = 0.02500000037252902984619140625;
    vec3 luma = vec3(0.2125999927520751953125, 0.715200006961822509765625, 0.072200000286102294921875);
    float lm2 = max(max(l2.x, l2.y), l2.z);
    float lm1 = max(max(l1.x, l1.y), l1.z);
    float rm1 = max(max(r1.x, r1.y), r1.z);
    float rm2 = max(max(r2.x, r2.y), r2.z);
    float swl2 = max(twl2, 0.0) * (dot(l2, luma) + ts);
    float swl1 = twl1 * (dot(l1, luma) + ts);
    float swr1 = twr1 * (dot(r1, luma) + ts);
    float swr2 = max(twr2, 0.0) * (dot(r2, luma) + ts);
    float fscolor1 = ((((lm2 * swl2) + (lm1 * swl1)) + (rm1 * swr1)) + (rm2 * swr2)) / (((swl2 + swl1) + swr1) + swr2);
    vec3 scolor1 = vec3(clamp(mix(max(max(color1.x, color1.y), color1.z), fscolor1, params.spike), 0.0, 1.0));
    vec3 color2;
    vec3 scolor2;
    if (!interb)
    {
        pC4 += offy;
        l3 = texture(LinearizePass, pC4 - off2).xyz;
        l2 = texture(LinearizePass, pC4 - offx).xyz;
        l1 = texture(LinearizePass, pC4).xyz;
        r1 = texture(LinearizePass, pC4 + offx).xyz;
        r2 = texture(LinearizePass, pC4 + off2).xyz;
        r3 = texture(LinearizePass, (pC4 + offx) + off2).xyz;
        colmin = min(min(l1, r1), min(l2, r2));
        colmax = max(max(l1, r1), max(l2, r2));
        if (smarte)
        {
            float pc_1 = min(global.smart_ei * c2.y, global.ei_limit);
            float pl_1 = min(global.smart_ei * max(c2.y, c2.x), global.ei_limit);
            float pr_1 = min(global.smart_ei * max(c2.y, c2.z), global.ei_limit);
            twl1 = max(wl1 - pc_1, 0.00999999977648258209228515625 * wl1);
            twr1 = max(wr1 - pc_1, 0.00999999977648258209228515625 * wr1);
            twl2 = max(wl2 - pl_1, 0.00999999977648258209228515625 * wl2);
            twr2 = max(wr2 - pr_1, 0.00999999977648258209228515625 * wr2);
        }
        color2 = ((((((l3 * twl3) + (l2 * twl2)) + (l1 * twl1)) + (r1 * twr1)) + (r2 * twr2)) + (r3 * twr3)) / vec3(((((twl3 + twl2) + twl1) + twr1) + twr2) + twr3);
        if (sharp)
        {
            color2 = clamp(color2, colmin, colmax);
        }
        lm2 = max(max(l2.x, l2.y), l2.z);
        lm1 = max(max(l1.x, l1.y), l1.z);
        rm1 = max(max(r1.x, r1.y), r1.z);
        rm2 = max(max(r2.x, r2.y), r2.z);
        swl2 = max(twl2, 0.0) * (dot(l2, luma) + ts);
        swl1 = twl1 * (dot(l1, luma) + ts);
        swr1 = twr1 * (dot(r1, luma) + ts);
        swr2 = max(twr2, 0.0) * (dot(r2, luma) + ts);
        float fscolor2 = ((((lm2 * swl2) + (lm1 * swl1)) + (rm1 * swr1)) + (rm2 * swr2)) / (((swl2 + swl1) + swr1) + swr2);
        scolor2 = vec3(clamp(mix(max(max(color2.x, color2.y), color2.z), fscolor2, params.spike), 0.0, 1.0));
    }
    vec3 ctmp = color1;
    float w3 = 1.0;
    vec3 color = color1;
    vec3 one = vec3(1.0);
    if (!interb)
    {
        float shape1 = mix(params.scanline1, params.scanline2, f);
        float shape2 = mix(params.scanline1, params.scanline2, 1.0 - f);
        float param_10 = f;
        float wt1 = st(param_10);
        float param_11 = 1.0 - f;
        float wt2 = st(param_11);
        vec3 color00 = (color1 * wt1) + (color2 * wt2);
        vec3 scolor0 = (scolor1 * wt1) + (scolor2 * wt2);
        ctmp = color00 / vec3(wt1 + wt2);
        vec3 sctmp = scolor0 / vec3(wt1 + wt2);
        vec3 cref1 = mix(sctmp, scolor1, vec3(params.beam_size));
        float creff1 = pow(max(max(cref1.x, cref1.y), cref1.z), global.scan_falloff);
        vec3 cref2 = mix(sctmp, scolor2, vec3(params.beam_size));
        float creff2 = pow(max(max(cref2.x, cref2.y), cref2.z), global.scan_falloff);
        float f1 = f;
        float f2 = 1.0 - f;
        float scanpix = mix(SourceSize.x / global.OutputSize.x, SourceSize.y / global.OutputSize.y, float(notate));
        f1 = fract(f1 + ((global.rolling_scan * float(global.FrameCount)) * scanpix));
        f2 = 1.0 - f1;
        float wf1;
        float wf2;
        if (params.gsl < 0.5)
        {
            float param_12 = f1;
            float param_13 = creff1;
            float param_14 = shape1;
            wf1 = sw0(param_12, param_13, param_14);
            float param_15 = f2;
            float param_16 = creff2;
            float param_17 = shape2;
            wf2 = sw0(param_15, param_16, param_17);
        }
        else
        {
            if (params.gsl == 1.0)
            {
                float param_18 = f1;
                float param_19 = creff1;
                float param_20 = shape1;
                float _1491 = sw1(param_18, param_19, param_20);
                wf1 = _1491;
                float param_21 = f2;
                float param_22 = creff2;
                float param_23 = shape2;
                float _1498 = sw1(param_21, param_22, param_23);
                wf2 = _1498;
            }
            else
            {
                float param_24 = f1;
                float param_25 = creff1;
                float param_26 = shape1;
                wf1 = sw2(param_24, param_25, param_26);
                float param_27 = f2;
                float param_28 = creff2;
                float param_29 = shape2;
                wf2 = sw2(param_27, param_28, param_29);
            }
        }
        if ((wf1 + wf2) > 1.0)
        {
            float wtmp = 1.0 / (wf1 + wf2);
            wf1 *= wtmp;
            wf2 *= wtmp;
        }
        vec3 w1 = vec3(wf1);
        vec3 w2 = vec3(wf2);
        w3 = wf1 + wf2;
        float mc1 = max(max(color1.x, color1.y), color1.z) + 1.0000000133514319600180897396058e-10;
        float mc2 = max(max(color2.x, color2.y), color2.z) + 1.0000000133514319600180897396058e-10;
        cref1 = color1 / vec3(mc1);
        cref1 *= cref1;
        cref1 *= cref1;
        cref2 = color2 / vec3(mc2);
        cref2 *= cref2;
        cref2 *= cref2;
        w1 = max(mix(w1 * mix(one, cref1, vec3(global.scans)), w1, vec3(wf1 * min(1.0 + (0.1500000059604644775390625 * global.scans), 1.2000000476837158203125))), vec3(0.0));
        w1 = min(w1 * color1, vec3(mc1)) / (color1 + vec3(1.0000000133514319600180897396058e-10));
        w2 = max(mix(w2 * mix(one, cref2, vec3(global.scans)), w2, vec3(wf2 * min(1.0 + (0.1500000059604644775390625 * global.scans), 1.2000000476837158203125))), vec3(0.0));
        w2 = min(w2 * color2, vec3(mc2)) / (color2 + vec3(1.0000000133514319600180897396058e-10));
        vec3 cd1 = one;
        vec3 cd2 = one;
        float vm = sqrt(abs(params.vertmask));
        float v_high1 = 1.0 + (0.300000011920928955078125 * vm);
        float v_high2 = 1.0 + (0.60000002384185791015625 * vm);
        float v_low = 1.0 - vm;
        float ds1 = min(max(1.0 - (w3 * w3), 2.5 * f1), 1.0);
        float ds2 = min(max(1.0 - (w3 * w3), 2.5 * f2), 1.0);
        if (params.vertmask < 0.0)
        {
            cd1 = mix(one, vec3(v_high2, v_low, v_low), vec3(ds1));
            cd2 = mix(one, vec3(v_low, v_high1, v_high1), vec3(ds2));
        }
        else
        {
            cd1 = mix(one, vec3(v_high1, v_low, v_high1), vec3(ds1));
            cd2 = mix(one, vec3(v_low, v_high2, v_low), vec3(ds2));
        }
        vec3 param_30 = color1;
        vec3 param_31 = color2;
        color = ((gc(param_30) * w1) * cd1) + ((gc(param_31) * w2) * cd2);
        if (abs(global.rolling_scan) > 0.004999999888241291046142578125)
        {
            float param_32 = f;
            wt1 = st1(param_32);
            float param_33 = 1.0 - f;
            wt2 = st1(param_33);
            color00 = ((color1 * wt1) + (color2 * wt2)) / vec3(wt1 + wt2);
            vec3 param_34 = color00;
            color = gc(param_34) * mix(w1 + w2, vec3(w3), vec3(max(wf1, wf2)));
        }
        color = min(color, vec3(1.0));
    }
    if (interb)
    {
        vec3 param_35 = color1;
        color = gc(param_35);
    }
    float colmx = pow(max(max(ctmp.x, ctmp.y), ctmp.z), 1.39999997615814208984375 / gamma_in);
    FragColor = vec4(color, colmx);
}

